  #define               W1_PIN              PINB2
  #define               W1_IN               PINB
  #define               W1_OUT              PORTB
  #define               W1_DDR              DDRB

  #define               MATCH_ROM           0x55
  #define               SKIP_ROM            0xCC
  #define               SEARCH_ROM          0xF0

  #define               CONVERT_T           0x44                // DS1820 commands
  #define               READ                0xBE
  #define               WRITE               0x4E
  #define               EE_WRITE            0x48
  #define               EE_RECALL           0xB8

  #define               SEARCH_FIRST        0xFF
  #define               PRESENCE_ERR        0xFF
  #define               DATA_ERR            0xFE
  #define               LAST_DEVICE         0x00


unsigned char One_Wire_RW_Bit( char b )
{
  asm cli;
  W1_DDR |= 1<<W1_PIN;
  Delay_us( 1 );
  if( b )
    W1_DDR &= ~(1<<W1_PIN);
  Delay_us( 15 - 1 );
  if( (W1_IN & (1<<W1_PIN)) == 0 )
    b = 0;
  Delay_us( 60 - 15 );
  W1_DDR &= ~(1<<W1_PIN);
  asm sei;
  return b;
}
char One_Wire_Reset(){
    char err;
    W1_OUT &= ~(1<<W1_PIN);          //Set PORTB2 = 0
    W1_DDR |= 1<<W1_PIN;             //Set DDRB2 as output
    Delay_us(480);                   //Wait for 480  sec
    asm cli;
    W1_DDR &= ~(1<<W1_PIN);          //Set DDRB2 as input
    Delay_us(66);                    //Wait for 66  sec
    err = (W1_IN & (1<<W1_PIN));     //No presence dedect
    asm sei;
    Delay_us( 480 - 66 );
    if( (W1_IN & (1<<W1_PIN)) == 0 ) // short circuit
      err = 1;
    return err;
}
unsigned int One_Wire_Write( unsigned char b )
{
  unsigned char i = 8, j;
  do{
    j = One_Wire_RW_Bit( b & 1 );
    b >>= 1;
    if( j )
      b |= 0x80;
  }while( --i );
  return b;
}

unsigned char One_Wire_Read( void )
{
  return One_Wire_Write( 0xFF );
}

unsigned char One_Wire_Search( unsigned char diff, unsigned char *id )
{
  unsigned char i, j, next_diff;
  char b;

  if( One_Wire_Reset() )
    return PRESENCE_ERR;                        // error, no device found
  One_Wire_Write( SEARCH_ROM );                        // ROM search command
  next_diff = LAST_DEVICE;                        // unchanged on last device
  i = 8 * 8;                                        // 8 bytes
  do{
    j = 8;                                        // 8 bits
    do{
      b = One_Wire_RW_Bit( 1 );                        // read bit
      if( One_Wire_RW_Bit( 1 ) ){                        // read complement bit
        if( b )                                        // 11
          return DATA_ERR;                        // data error
      }else{
        if( !b ){                                // 00 = 2 devices
          if( diff > i ||
            ((*id & 1) && diff != i) ){
            b = 1;                                // now 1
            next_diff = i;                        // next pass 0
          }
        }
      }
      One_Wire_RW_Bit( b );                             // write bit
      *id >>= 1;
      if( b )                                        // store bit
        *id |= 0x80;
      i--;
    }while( --j );
    id++;                                        // next byte
  }while( i );
  return next_diff;                                // to continue search
}

void One_Wire_Command( unsigned char command, unsigned char *id )
{
  unsigned char i;
  One_Wire_Reset();
  if( id ){
    One_Wire_Write( MATCH_ROM );                        // to a single device
    i = 8;
    do{
      One_Wire_Write( *id );
      id++;
    }while( --i );
  }else{
    One_Wire_Write( SKIP_ROM );                        // to all devices
  }
  One_Wire_Write( command );
}